@@ -40,42 +40,27 @@
 #ifndef QBACKINGSTORE_COCOA_H
 #define QBACKINGSTORE_COCOA_H
 
-#include <QtGraphicsSupport/private/qrasterbackingstore_p.h>
+#include <QtGui/private/qrasterbackingstore_p.h>
 
 #include <private/qcore_mac_p.h>
 
 #include <QScopedPointer>
 #include "qiosurfacegraphicsbuffer.h"
 
+#include <unordered_map>
+
 QT_BEGIN_NAMESPACE
 
-class QCocoaBackingStore : public QRasterBackingStore
+class QCocoaBackingStore : public QPlatformBackingStore
 {
 protected:
     QCocoaBackingStore(QWindow *window);
     QCFType<CGColorSpaceRef> colorSpace() const;
-    QMacNotificationObserver m_backingPropertiesObserver;
-    virtual void backingPropertiesChanged() = 0;
 };
 
-class QNSWindowBackingStore : public QCocoaBackingStore
-{
-public:
-    QNSWindowBackingStore(QWindow *window);
-    ~QNSWindowBackingStore();
-
-    void resize(const QSize &size, const QRegion &staticContents) override;
-    void flush(QWindow *, const QRegion &, const QPoint &) override;
-
-private:
-    bool windowHasUnifiedToolbar() const;
-    QImage::Format format() const override;
-    void redrawRoundedBottomCorners(CGRect) const;
-    void backingPropertiesChanged() override;
-};
-
-class QCALayerBackingStore : public QCocoaBackingStore
+class QCALayerBackingStore : public QObject, public QCocoaBackingStore
 {
+    Q_OBJECT
 public:
     QCALayerBackingStore(QWindow *window);
     ~QCALayerBackingStore();
@@ -96,6 +81,9 @@ public:
     QPlatformGraphicsBuffer *graphicsBuffer() const override;
 
 private:
+    void observeBackingPropertiesChanges();
+    bool eventFilter(QObject *watched, QEvent *event) override;
+
     QSize m_requestedSize;
     QRegion m_paintedRegion;
 
@@ -118,9 +106,15 @@ private:
     bool recreateBackBufferIfNeeded();
     bool prepareForFlush();
 
-    void backingPropertiesChanged() override;
+    void backingPropertiesChanged();
+    QMacNotificationObserver m_backingPropertiesObserver;
 
     std::list<std::unique_ptr<GraphicsBuffer>> m_buffers;
+
+    void flushSubWindow(QWindow *window);
+    std::unordered_map<QWindow*, std::unique_ptr<QCALayerBackingStore>> m_subWindowBackingstores;
+    void windowDestroyed(QObject *object);
+    bool m_clearSurfaceOnPaint = true;
 };
 
 QT_END_NAMESPACE
