@@ -26,14 +26,16 @@
 **
 ****************************************************************************/
 
-
-#include <QtTest/QtTest>
-#include <qapplication.h>
-#include <qtabbar.h>
-
-#include <qpushbutton.h>
-#include <qstyle.h>
-#include <qstyleoption.h>
+#include <QTest>
+#include <QSignalSpy>
+#include <QApplication>
+#include <QTabBar>
+#include <QPushButton>
+#include <QLabel>
+#include <QStyle>
+#include <QStyleOptionTab>
+#include <QProxyStyle>
+#include <QTimer>
 
 class TabBar;
 
@@ -98,6 +100,11 @@ private slots:
 
     void mouseReleaseOutsideTabBar();
 
+    void mouseWheel();
+
+    void scrollButtons_data();
+    void scrollButtons();
+
 private:
     void checkPositions(const TabBar &tabbar, const QList<int> &positions);
 };
@@ -869,5 +876,128 @@ void tst_QTabBar::checkPositions(const TabBar &tabbar, const QList<int> &positio
     }
 }
 
+#if QT_CONFIG(wheelevent)
+// defined to be 120 by the wheel mouse vendors according to the docs
+#define WHEEL_DELTA 120
+
+class TabBarScrollingProxyStyle : public QProxyStyle
+{
+public:
+    TabBarScrollingProxyStyle() : QProxyStyle(), scrolling(true)
+    { }
+
+    int styleHint(StyleHint hint, const QStyleOption *option = 0,
+                  const QWidget *widget = 0, QStyleHintReturn *returnData = 0) const override
+    {
+        if (hint == QStyle::SH_TabBar_AllowWheelScrolling)
+            return scrolling;
+
+        return QProxyStyle::styleHint(hint, option, widget, returnData);
+    }
+
+    bool scrolling;
+};
+
+void tst_QTabBar::mouseWheel()
+{
+
+    // apply custom style to app, which can toggle tabbar scrolling behavior
+    QCoreApplication *applicationInstance = QApplication::instance();
+    QVERIFY(applicationInstance != 0);
+    auto *proxyStyle = new TabBarScrollingProxyStyle;
+    QApplication::setStyle(proxyStyle);
+
+    // make tabbar with three tabs, select the middle one
+    TabBar tabbar;
+    tabbar.addTab("one");
+    tabbar.addTab("two");
+    tabbar.addTab("three");
+    int startIndex = 1;
+    tabbar.setCurrentIndex(startIndex);
+
+    // define scroll event
+    const QPoint wheelPoint = tabbar.rect().bottomRight();
+    QWheelEvent event(wheelPoint, tabbar.mapToGlobal(wheelPoint), QPoint(), QPoint(0, WHEEL_DELTA),
+                      Qt::NoButton, Qt::NoModifier, Qt::NoScrollPhase, false);
+
+    // disable scrolling, send scroll event, confirm that tab did not change
+    proxyStyle->scrolling = false;
+    QVERIFY(applicationInstance->sendEvent(&tabbar, &event));
+    QVERIFY(tabbar.currentIndex() == startIndex);
+
+    // enable scrolling, send scroll event, confirm that tab changed
+    proxyStyle->scrolling = true;
+    QVERIFY(applicationInstance->sendEvent(&tabbar, &event));
+    QVERIFY(tabbar.currentIndex() != startIndex);
+}
+
+void tst_QTabBar::scrollButtons_data()
+{
+    QTest::addColumn<QTabWidget::TabPosition>("tabPosition");
+    QTest::addColumn<Qt::LayoutDirection>("layoutDirection");
+
+    for (auto ld : {Qt::LeftToRight, Qt::RightToLeft}) {
+        const char *ldStr = ld == Qt::LeftToRight ? "LTR" : "RTL";
+        QTest::addRow("North, %s", ldStr) << QTabWidget::North << ld;
+        QTest::addRow("South, %s", ldStr) << QTabWidget::South << ld;
+        QTest::addRow("West, %s", ldStr) << QTabWidget::West << ld;
+        QTest::addRow("East, %s", ldStr) << QTabWidget::East << ld;
+    }
+}
+
+void tst_QTabBar::scrollButtons()
+{
+    QFETCH(QTabWidget::TabPosition, tabPosition);
+    QFETCH(Qt::LayoutDirection, layoutDirection);
+
+    QWidget window;
+    QTabWidget tabWidget(&window);
+    tabWidget.setLayoutDirection(layoutDirection);
+    tabWidget.setTabPosition(tabPosition);
+    tabWidget.setElideMode(Qt::ElideNone);
+    tabWidget.setUsesScrollButtons(true);
+
+    const int tabCount = 5;
+    for (int i = 0; i < tabCount; ++i)
+    {
+        const QString num = QString::number(i);
+        tabWidget.addTab(new QLabel(num), num + " - Really long tab name to force arrows");
+    }
+    tabWidget.move(0, 0);
+    tabWidget.resize(tabWidget.minimumSizeHint());
+    window.show();
+    QVERIFY(QTest::qWaitForWindowActive(&window));
+
+    auto *leftB = tabWidget.tabBar()->findChild<QAbstractButton*>(u"ScrollLeftButton"_qs);
+    auto *rightB = tabWidget.tabBar()->findChild<QAbstractButton*>(u"ScrollRightButton"_qs);
+
+    QVERIFY(leftB->isVisible());
+    QVERIFY(!leftB->isEnabled());
+    QVERIFY(rightB->isVisible());
+    QVERIFY(rightB->isEnabled());
+    QVERIFY(!tabWidget.tabBar()->tabRect(1).intersects(tabWidget.tabBar()->rect()));
+
+    int index = 0;
+    for (; index < tabWidget.count(); ++index) {
+        QCOMPARE(leftB->isEnabled(), index > 0);
+        QCOMPARE(rightB->isEnabled(), index < tabWidget.count() - 1);
+        QVERIFY(tabWidget.tabBar()->tabRect(index).intersects(tabWidget.tabBar()->rect()));
+        QCOMPARE(tabWidget.tabBar()->tabAt(tabWidget.tabBar()->rect().center()), index);
+        if (rightB->isEnabled())
+            rightB->click();
+    }
+    for (--index; index >= 0; --index) {
+        QCOMPARE(leftB->isEnabled(), index >= 0);
+        QCOMPARE(rightB->isEnabled(), index < tabWidget.count() - 1);
+
+        QVERIFY(tabWidget.tabBar()->tabRect(index).intersects(tabWidget.tabBar()->rect()));
+        if (leftB->isEnabled())
+            leftB->click();
+    }
+    QVERIFY(!leftB->isEnabled());
+}
+
+#endif // QT_CONFIG(wheelevent)
+
 QTEST_MAIN(tst_QTabBar)
 #include "tst_qtabbar.moc"
